import os
from PIL import Image, ImageDraw, ImageFont
import unicodedata

# Constants
DATA_PATH = '/home/mark/Projects/glyph-generator/data2'
OUTPUT_PATH = '/home/mark/Pictures/dupes'

F = 31
FONT_PATH = '/usr/share/fonts/truetype/noto/NotoSansZanabazarSquare-Regular.ttf'
UNICODE_START = 0x11a0b
UNICODE_END = 0x11a46
FONT_SIZE = 16
VERT_OFFSET = -10
HORZ_OFFSET = 0

AUTO_CENTRE = True
GLYPH_SIZE = 16
SCALE = 1
THRESHOLD = 128  # Threshold for converting to black-and-white

def center_glyph(image):
    """Centers the glyph within the given image."""
    bbox = image.getbbox()  # Get bounding box of the non-zero pixels
    if bbox:
        glyph_width = bbox[2] - bbox[0]
        glyph_height = bbox[3] - bbox[1]

        x_offset = (GLYPH_SIZE - glyph_width) // 2 - bbox[0]
        y_offset = (GLYPH_SIZE - glyph_height) // 2 - bbox[1]

        centered_image = Image.new('L', (GLYPH_SIZE, GLYPH_SIZE), color=0)
        centered_image.paste(image, (x_offset, y_offset))
        return centered_image
    return image  # Return the original if the bounding box is empty

def unicode_to_image(unicode_char, font, output_file):
    """Converts a Unicode character to an image."""
    # Create a blank image for the glyph
    img = Image.new('L', (GLYPH_SIZE, GLYPH_SIZE), color=0)  # Black background
    draw = ImageDraw.Draw(img)
    draw.text((HORZ_OFFSET, VERT_OFFSET), unicode_char, fill=255, font=font)

    # Apply auto-centering if enabled
    if AUTO_CENTRE:
        img = center_glyph(img)

    # Convert to black-and-white using a threshold
    img = img.point(lambda p: 255 if p > THRESHOLD else 0, mode='1')  # type: ignore

    # Upscale if needed
    if SCALE != 1:
        upscale_size = GLYPH_SIZE * SCALE
        img_upscale = img.resize((upscale_size, upscale_size), resample=Image.NEAREST)  # type: ignore
    else:
        img_upscale = img

    os.makedirs(OUTPUT_PATH, exist_ok=True)
    img_upscale.save(output_file)
    print(f'Saved: {output_file}')

def is_printable(char):
    """Checks if a character is printable."""
    try:
        category = unicodedata.category(char)
        return not category.startswith(('C'))  # Exclude control characters
    except ValueError:
        return False

def generate_unicode_glyphs():
    """Generates glyphs for a range of Unicode characters."""
    os.makedirs(OUTPUT_PATH, exist_ok=True)
    font = ImageFont.truetype(FONT_PATH, size=FONT_SIZE)
    for codepoint in range(UNICODE_START, UNICODE_END + 1):
        unicode_char = chr(codepoint)
        if is_printable(unicode_char):  # Check if the character is printable
            output_file = f'{OUTPUT_PATH}/F{F}_{codepoint}.jpg'
            unicode_to_image(unicode_char, font, output_file)
        else:
            print(f'Skipped non-printable character: U+{codepoint:04X}')

# Generate Unicode glyphs
generate_unicode_glyphs()
